#
# Generic Screen to select a path/filename combination
#

# GUI (Screens)
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.HelpMenu import HelpableScreen
from Screens.ChoiceBox import ChoiceBox

# Generic
from Tools.BoundFunction import boundFunction
from Tools.Directories import *

# Quickselect
from Tools.NumericalTextInput import NumericalTextInput

# GUI (Components)
from Components.ActionMap import NumberActionMap, HelpableActionMap
from Components.Label import Label
from Components.Pixmap import Pixmap
from Components.Button import Button
from Components.FileList import FileList
from Components.MenuList import MenuList

# Timer
from enigma import eTimer

from Components.config import config, configfile, ConfigSubList, ConfigSubsection, ConfigText, ConfigNumber, ConfigBoolean

config.locationbox = ConfigSubsection()
config.locationbox.usermode = ConfigBoolean(default = False)
config.misc.bookmarks = ConfigSubsection()
config.misc.bookmarks.count = ConfigNumber(default = 0)
config.misc.bookmarks.list = ConfigSubList()
for i in range(0, config.misc.bookmarks.count.value):
	config.misc.bookmarks.list.append(ConfigSubsection())
	config.misc.bookmarks.list[i].name = ConfigText(default = "", fixed_size = False)
	config.misc.bookmarks.list[i].dir = ConfigText(default = "", fixed_size = False)

class LocationBox(Screen, NumericalTextInput, HelpableScreen):
	"""Simple Class similar to MessageBox / ChoiceBox but used to choose a folder/pathname combination"""

	skin = """<screen name="LocationBox" position="100,75" size="540,460" >
			<widget name="text" position="0,2" size="540,22" font="Regular;22" />
			<widget name="target" position="0,23" size="540,22" valign="center" font="Regular;22" />
			<widget name="filelist" position="0,55" zPosition="1" size="540,210" scrollbarMode="showOnDemand" selectionDisabled="1" />
			<widget name="textbook" position="0,272" size="540,22" font="Regular;22" />
			<widget name="booklist" position="5,302" zPosition="2" size="535,100" scrollbarMode="showOnDemand" />
			<widget name="red" position="0,415" zPosition="1" size="135,40" pixmap="skin_default/key-red.png" transparent="1" alphatest="on" />
			<widget name="key_red" position="0,415" zPosition="2" size="135,40" halign="center" valign="center" font="Regular;22" transparent="1" shadowColor="black" shadowOffset="-1,-1" />	
			<widget name="green" position="135,415" zPosition="1" size="135,40" pixmap="skin_default/key-green.png" transparent="1" alphatest="on" />
			<widget name="key_green" position="135,415" zPosition="2" size="135,40" halign="center" valign="center" font="Regular;22" transparent="1" shadowColor="black" shadowOffset="-1,-1" />
			<widget name="yellow" position="270,415" zPosition="1" size="135,40" pixmap="skin_default/key-yellow.png" transparent="1" alphatest="on" />
			<widget name="key_yellow" position="270,415" zPosition="2" size="135,40" halign="center" valign="center" font="Regular;22" transparent="1" shadowColor="black" shadowOffset="-1,-1" />
			<widget name="blue" position="405,415" zPosition="1" size="135,40" pixmap="skin_default/key-blue.png" transparent="1" alphatest="on" />
			<widget name="key_blue" position="405,415" zPosition="2" size="135,40" halign="center" valign="center" font="Regular;22" transparent="1" shadowColor="black" shadowOffset="-1,-1" />			
		</screen>"""

	def __init__(self, session, text = "", filename = "", currDir = None, windowTitle = _("Select Location"), minFree = None):
		# Init parents
		Screen.__init__(self, session)
		NumericalTextInput.__init__(self, handleTimeout = False)
		HelpableScreen.__init__(self)

		# Set useable chars
		self.setUseableChars(u'1234567890abcdefghijklmnopqrstuvwxyz')

		# Quickselect Timer
		self.qs_timer = eTimer()
		self.qs_timer.callback.append(self.timeout)
		self.qs_timer_type = 0

		# Initialize Quickselect
		self.curr_pos = -1
		self.quickselect = ""

		# Set Text
		self["text"] = Label(text)
		self["textbook"] = Label("Bookmarks")

		# Save parameters locally
		self.text = text
		self.filename = filename
		self.minFree = minFree
		
		self.folder = ""
		self.saveflag = False
		
		# Initialize FileList
		self["filelist"] = FileList(currDir, showDirectories = True, showFiles = False)
		
		# Initialize BookList
		self["booklist"] = MenuList([ ])
		if self.filename == "":
			self.loadBooklist()

		# Buttons
		self["key_green"] = Button(_("ok"))
		self["key_yellow"] = Button(_("Rename"))
		self["key_blue"] = Button(_("Remove Bookmark"))
		self["key_red"] = Button(_("Cancel"))

		# Background for Buttons
		self["green"] = Pixmap()
		self["yellow"] = Pixmap()
		self["blue"] = Pixmap()
		self["red"] = Pixmap()

		# Initialize Target
		self["target"] = Label()

		if config.locationbox.usermode.value == True:
			self.usermodeOn()

		# Custom Action Handler
		class LocationBoxActionMap(NumberActionMap):
			def __init__(self, box, contexts = [ ], actions = { }, prio=0):
				NumberActionMap.__init__(self, contexts, actions, prio)
				self.box = box

			def action(self, contexts, action):
				# Reset Quickselect
				self.box.timeout(force = True)

				return NumberActionMap.action(self, contexts, action)

		# Actions that will reset quickselect
		self["actions"] = LocationBoxActionMap(self, ["WizardActions", "ColorActions"],
		{
			"red": self.cancel,
			"green": self.select,
			"yellow": self.changeName,
			"blue": self.AddRemoveBookmark,
			"left": self.left,
			"right": self.right,
			"up": self.up,
			"down": self.down,
		}, -2)

		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions", 
			{
				"ok": (self.ok, _("select")),
				"cancel": (self.cancel, _("cancel")),
			}, -2)
		
		self["EPGSelectActions"] = HelpableActionMap(self, "EPGSelectActions",
			{
				"prevBouquet": (self.switchToBookList, _("switch to booklist")),
				"nextBouquet": (self.switchToFileList, _("switch to filelist")),
			}, -2)

		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
			{
				"menu": (self.showMenu, _("menu")),
			}, -2)

		# Actions used by quickselect
		self["NumberActions"] = NumberActionMap(["NumberActions"],
		{
			"1": self.keyNumberGlobal,
			"2": self.keyNumberGlobal,
			"3": self.keyNumberGlobal,
			"4": self.keyNumberGlobal,
			"5": self.keyNumberGlobal,
			"6": self.keyNumberGlobal,
			"7": self.keyNumberGlobal,
			"8": self.keyNumberGlobal,
			"9": self.keyNumberGlobal,
			"0": self.keyNumberGlobal
		})

		# Run some functions when shown
		self.inited = False
		self.onShown.extend([
			boundFunction(self.setTitle, windowTitle),
			self.updateTarget,
			self.showHideRename,
			self.switchToFileListOnStart
		])
 
		# Make sure we remove our callback
		self.onClose.append(self.disableTimer)

	def switchToFileListOnStart(self):
		if not self.inited:
			print "onShown LocationBox inited"
			self.inited=True
			if self.filename == "":
				if config.misc.bookmarks.count.value > 0:
					self.currList = "booklist"
				else:
					self.switchToFileList()
			else:
				self.switchToFileList()

	def disableTimer(self):
		self.qs_timer.callback.remove(self.timeout)

	def showHideRename(self):
		# Don't allow renaming when filename is empty
		if self.filename == "":
			self["key_yellow"].hide()

	def up(self):
		self[self.currList].up()

	def down(self):
		self[self.currList].down()

	def left(self):
		self[self.currList].pageUp()

	def right(self):
		self[self.currList].pageDown()

	def switchToFileList(self):
		if config.locationbox.usermode.value == False:
			self.currList = "filelist"
			self["filelist"].selectionEnabled(1)
			self["booklist"].selectionEnabled(0)
			self["key_blue"].text = _("Add Bookmark")

	def switchToBookList(self):
		# Don't allow switch to booklist when filename is empty
		if self.filename == "":
			self.currList = "booklist"
			self["filelist"].selectionEnabled(0)
			self["booklist"].selectionEnabled(1)
			self["key_blue"].text = _("Remove Bookmark")

	def AddRemoveBookmark(self):
		if self.currList == "filelist":
			if self.filename == "":
				# add bookmark
				self.folder = self["filelist"].getSelection()[0]
				if self.folder is not None:
					self.session.openWithCallback(
						self.addBookmarkCallback,
						InputBox,
						title = _("Enter a name for the new Bookmark"),
						text = self.folder
					)
		else:
			# remove bookmark
			if config.locationbox.usermode.value == False:
				name = self["booklist"].getCurrent()[0]
				flag = False
				for i in range(0, config.misc.bookmarks.count.value):
					if config.misc.bookmarks.list[i].name.value == name:
						flag = True
						break
				if flag == False:
					self.session.open(MessageBox,_("default recorded Movies can not removed"), MessageBox.TYPE_INFO)
				else:
					self.session.openWithCallback(
					boundFunction(self.removeBookmark, i),
					MessageBox,
					_("Do you really want to remove your bookmark of %s?") % (name),
				)

	def removeBookmark(self, i, ret):
		if not ret:
			return
		
		del config.misc.bookmarks.list[i]
		config.misc.bookmarks.count.value -= 1
		config.misc.bookmarks.save()
		self.saveflag = True
		self.loadBooklist()

	def addBookmarkCallback(self, ret):
		if ret:
			i = config.misc.bookmarks.count.value
			config.misc.bookmarks.list.append(ConfigSubsection())
			config.misc.bookmarks.list[i].name = ConfigText(default = "", fixed_size = False)
			config.misc.bookmarks.list[i].dir = ConfigText(default = "", fixed_size = False)
			config.misc.bookmarks.list[i].name.value = ret
			config.misc.bookmarks.list[i].dir.value = self.folder
			config.misc.bookmarks.count.value += 1
			config.misc.bookmarks.save()
			self.saveflag = True
			self.loadBooklist()
			
	def loadBooklist(self):
		booklist = [(_("default recorded Movies"), defaultPaths[SCOPE_HDD][0])]
		if config.misc.bookmarks.count.value > 0:
			for i in range(0, config.misc.bookmarks.count.value):
				booklist.append((config.misc.bookmarks.list[i].name.value, config.misc.bookmarks.list[i].dir.value))
		self["booklist"].setList(booklist)
		
	def saveconfig(self):
		if self.saveflag == True:
			configfile.save()

	def ok(self):
		if self.currList == "filelist":
			if self["filelist"].canDescent():
				self["filelist"].descent()
				self.updateTarget()
		else:
			self.saveconfig()
			self.close(self["booklist"].getCurrent()[1])

	def cancel(self):
		self.saveconfig()
		self.close(None)

	def selectConfirmed(self, res):
		if res: 
			self.saveconfig()
			self.close(''.join([self["filelist"].getCurrentDirectory(), self.filename]))

	def select(self):
		if self.currList == "filelist":
			# Do nothing unless current Directory is valid
			if self["filelist"].getCurrentDirectory() is not None:
				# Check if we need to have a minimum of free Space available
				if self.minFree is not None:
					# Try to read fs stats
					try:
						from os import statvfs

						s = statvfs(self["filelist"].getCurrentDirectory())
						if (s.f_bavail * s.f_bsize) / 1000000 > self.minFree:
							# Automatically confirm if we have enough free disk Space available
							return self.selectConfirmed(True)
					except OSError:
						pass

					# Ask User if he really wants to select this folder
					self.session.openWithCallback(
						self.selectConfirmed,
						MessageBox,
						_("There might not be enough Space on the selected Partition.\nDo you really want to continue?"),
						type = MessageBox.TYPE_YESNO
					)
					# No minimum free Space means we can safely close
				else:
					self.selectConfirmed(True)
		else:
			self.saveconfig()
			self.close(self["booklist"].getCurrent()[1])

	def changeName(self):
		if self.filename != "":
			# TODO: Add Information that changing extension is bad? disallow?
			# TODO: decide if using an inputbox is ok - we could also keep this in here
			self.session.openWithCallback(
				self.nameChanged,
				InputBox,
				title = _("Please enter a new filename"),
				text = self.filename
			)

	def nameChanged(self, res):
		if res is not None:
			if len(res):
				self.filename = res
				self.updateTarget()
			else:
				self.session.open(
					MessageBox,
					_("An empty filename is illegal."),
					type = MessageBox.TYPE_ERROR,
					timeout = 5
				)

	def updateTarget(self):
		# Write Combination of Folder & Filename when Folder is valid
		if self["filelist"].getCurrentDirectory() is not None:
			self["target"].setText(''.join([self["filelist"].getCurrentDirectory(), self.filename]))
		# Warning else
		else:
			self["target"].setText(_("Invalid Location"))

	def showMenu(self):
		if self.filename == "":
			menu = []
			if config.locationbox.usermode.value == False:
				if self.currList == "filelist":
					menu.append((_("switch to booklist"), "booklist"))
				else:
					menu.append((_("switch to filelist"), "filelist"))
				menu.append((_("usermode on"), "usermodeOn"))
			else:
				menu.append((_("usermode off"), "usermodeOff"))
			self.session.openWithCallback(self.menuCallback, ChoiceBox, title="", list=menu)

	def menuCallback(self, choice):
		if choice is None:
			return
		if choice[1] == "booklist":
			self.switchToBookList()
		elif choice[1] == "filelist":
			self.switchToFileList()
		elif choice[1] == "usermodeOn":
			config.locationbox.usermode.value = True
			config.locationbox.usermode.save()
			self.saveflag = True
			self.usermodeOn()
		elif choice[1] == "usermodeOff":
			config.locationbox.usermode.value = False
			config.locationbox.usermode.save()
			self.saveflag = True
			self["filelist"].show()
			self["key_blue"].show()
			
	def usermodeOn(self):
		if self.filename == "":
			self.switchToBookList()
			self["filelist"].hide()
			self["key_blue"].hide()

	def keyNumberGlobal(self, number):
		# Cancel Timeout
		self.qs_timer.stop()

		# See if another key was pressed before
		if number != self.lastKey:
			# Reset lastKey again so NumericalTextInput triggers its keychange
			self.nextKey()

			# Try to select what was typed
			self.selectByStart()

			# Increment position
			self.curr_pos += 1

		# Get char and append to text
		char = self.getKey(number)
		self.quickselect = self.quickselect[:self.curr_pos] + unicode(char)

		# Start Timeout
		self.qs_timer_type = 0
		self.qs_timer.start(1000, 1)

	def selectByStart(self):
		# Don't do anything on initial call
		if not len(self.quickselect):
			return

		# Don't select if no dir
		if self["filelist"].getCurrentDirectory():
			# TODO: implement proper method in Components.FileList
			files = self["filelist"].getFileList()

			# Initialize index
			idx = 0

			# We select by filename which is absolute
			lookfor = self["filelist"].getCurrentDirectory() + self.quickselect

			# Select file starting with generated text
			for file in files:
				if file[0][0] and file[0][0].lower().startswith(lookfor):
					self["filelist"].instance.moveSelectionTo(idx)
					break
				idx += 1

	def timeout(self, force = False):
		# Timeout Key
		if not force and self.qs_timer_type == 0:
			# Try to select what was typed
			self.selectByStart()

			# Reset Key
			self.lastKey = -1

			# Change type
			self.qs_timer_type = 1

			# Start timeout again
			self.qs_timer.start(1000, 1)
		# Timeout Quickselect
		else:
			# Eventually stop Timer
			self.qs_timer.stop()

			# Invalidate
			self.lastKey = -1
			self.curr_pos = -1
			self.quickselect = ""

	def __repr__(self):
		return str(type(self)) + "(" + self.text + ")"

